﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2013 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using HeuristicLab.DataImporter.Command.View;
using HeuristicLab.DataImporter.Data;
using HeuristicLab.DataImporter.Data.CommandBase;
using HeuristicLab.DataImporter.Data.Model;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.DataImporter.Command {
  [StorableClass]
  [ViewableCommandInfoAttribute("Moving Median", 1, ColumnGroupState.DoubleColumnSelected, "Change Values", Position = 3,
   OptionsView = typeof(FilterCommandView))]
  public class FilterMovingMedianCommand : FilterCommandBase {
    private Dictionary<int, DoubleColumn> oldColumns;
    private ICollection<int> oldSortedColumnIndices;

    [StorableConstructor]
    protected FilterMovingMedianCommand(bool deserializing)
      : base(deserializing) {
      oldColumns = new Dictionary<int, DoubleColumn>();
    }

    public FilterMovingMedianCommand(DataSet dataSet, string columnGroupName, int[] affectedColumns)
      : base(dataSet, columnGroupName, affectedColumns) {
      oldColumns = new Dictionary<int, DoubleColumn>();
    }

    public override string Description {
      get { return "Filter Column - Moving Median"; }
    }

    public override void Execute() {
      base.Execute();
      if (this.WindowSize < 1)
        throw new CommandExecutionException("Window size for filter commands must no be smaller than 1.", this);
      oldSortedColumnIndices = new List<int>(ColumnGroup.SortedColumnIndexes);
      DoubleColumn column;
      foreach (int col in AffectedColumns) {
        if (ColumnGroup.GetColumn(col) is DoubleColumn) {
          column = (DoubleColumn)ColumnGroup.GetColumn(col);
          oldColumns.Add(col, column);
          ColumnGroup.ReplaceColumn(col, CalcNewColumn(column, this.WindowSize));
        }
      }
      ColumnGroup.SortedColumnIndexes = oldSortedColumnIndices;
      ColumnGroup.FireChanged();
      ColumnGroup = null;
    }

    public override void UndoExecute() {
      base.UndoExecute();
      foreach (KeyValuePair<int, DoubleColumn> pair in oldColumns)
        ColumnGroup.ReplaceColumn(pair.Key, pair.Value);
      ColumnGroup.SortedColumnIndexes = oldSortedColumnIndices;
      oldSortedColumnIndices = null;
      oldColumns.Clear();
      ColumnGroup.FireChanged();
      ColumnGroup = null;
    }

    private DoubleColumn CalcNewColumn(DoubleColumn oldColumn, int windowSize) {
      DoubleColumn newCol = (DoubleColumn)oldColumn.CreateCopyOfColumnWithoutValues();
      int left = windowSize / 2;
      int right = windowSize / 2;
      if (windowSize % 2 == 0) left--;

      int startIndex = -1;
      int endIndex = oldColumn.TotalValuesCount;
      //first value newColumn[0]
      newCol.AddValue(oldColumn.GetMedian(0, right >= oldColumn.TotalValuesCount ? oldColumn.TotalValuesCount : right + 1));
      for (int i = 1; i < oldColumn.TotalValuesCount; i++) {
        startIndex = i - left;
        if (startIndex < 0)
          startIndex = 0;
        endIndex = i + right;
        if (endIndex >= oldColumn.TotalValuesCount)
          endIndex = oldColumn.TotalValuesCount - 1;
        newCol.AddValue(oldColumn.GetMedian(startIndex, endIndex + 1));
      }
      newCol.SortOrder = oldColumn.SortOrder;
      return newCol;
    }
  }
}
